@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 import sys
+import setup_test
 import libxml2
 try:
     import StringIO
@@ -26,5 +27,4 @@ if libxml2.debugMemory(1) == 0:
     print("OK")
 else:
     print("Memory leak %d bytes" % (libxml2.debugMemory(1)))
-    libxml2.dumpMemory()
 
