@@ -3,7 +3,7 @@
  *
  * See Copyright for the status of this software.
  *
- * daniel@veillard.com
+ * Author: Daniel Veillard
  */
 
 #include "libxml.h"
@@ -29,10 +29,10 @@
 static FILE *logfile = NULL;
 static int verbose = 0;
 
-#define NB_EXPECTED_ERRORS 15
+#define NB_EXPECTED_ERRORS 5
 
 
-const char *skipped_tests[] = {
+static const char *const skipped_tests[] = {
 /* http://lists.w3.org/Archives/Public/public-xml-testsuite/2008Jul/0000.html */
     "rmt-ns10-035",
     NULL
@@ -178,6 +178,8 @@ xmlconfTestInvalid(const char *id, const char *filename, int options) {
     if (doc == NULL) {
         test_log("test %s : %s invalid document turned not well-formed too\n",
 	         id, filename);
+        nb_errors++;
+        ret = 0;
     } else {
     /* invalidity should be reported both in the context and in the document */
         if ((ctxt->valid != 0) || (doc->properties & XML_DOC_DTDVALID)) {
