@@ -3,7 +3,7 @@
  *
  * See Copyright for the status of this software.
  *
- * Daniel Veillard <daniel@veillard.com>
+ * Author: Daniel Veillard
  */
 
 /*
@@ -69,8 +69,6 @@ typedef enum {
 } xmlSchematronTestType;
 
 /**
- * _xmlSchematronLet:
- *
  * A Schematron let variable
  */
 typedef struct _xmlSchematronLet xmlSchematronLet;
@@ -82,8 +80,6 @@ struct _xmlSchematronLet {
 };
 
 /**
- * _xmlSchematronTest:
- *
  * A Schematrons test, either an assert or a report
  */
 typedef struct _xmlSchematronTest xmlSchematronTest;
@@ -98,8 +94,6 @@ struct _xmlSchematronTest {
 };
 
 /**
- * _xmlSchematronRule:
- *
  * A Schematrons rule
  */
 typedef struct _xmlSchematronRule xmlSchematronRule;
@@ -116,8 +110,6 @@ struct _xmlSchematronRule {
 };
 
 /**
- * _xmlSchematronPattern:
- *
  * A Schematrons pattern
  */
 typedef struct _xmlSchematronPattern xmlSchematronPattern;
@@ -129,8 +121,6 @@ struct _xmlSchematronPattern {
 };
 
 /**
- * _xmlSchematron:
- *
  * A Schematrons definition
  */
 struct _xmlSchematron {
@@ -155,8 +145,6 @@ struct _xmlSchematron {
 };
 
 /**
- * xmlSchematronValidCtxt:
- *
  * A Schematrons validation context
  */
 struct _xmlSchematronValidCtxt {
@@ -225,11 +213,9 @@ struct _xmlSchematronParserCtxt {
  ************************************************************************/
 
 /**
- * xmlSchematronPErrMemory:
- * @node: a context node
- * @extra:  extra information
- *
  * Handle an out of memory condition
+ *
+ * @param ctxt  parser context
  */
 static void
 xmlSchematronPErrMemory(xmlSchematronParserCtxtPtr ctxt)
@@ -240,15 +226,14 @@ xmlSchematronPErrMemory(xmlSchematronParserCtxtPtr ctxt)
 }
 
 /**
- * xmlSchematronPErr:
- * @ctxt: the parsing context
- * @node: the context node
- * @error: the error code
- * @msg: the error message
- * @str1: extra data
- * @str2: extra data
- *
  * Handle a parser error
+ *
+ * @param ctxt  the parsing context
+ * @param node  the context node
+ * @param error  the error code
+ * @param msg  the error message
+ * @param str1  extra data
+ * @param str2  extra data
  */
 static void LIBXML_ATTR_FORMAT(4,0)
 xmlSchematronPErr(xmlSchematronParserCtxtPtr ctxt, xmlNodePtr node, int error,
@@ -280,11 +265,9 @@ xmlSchematronPErr(xmlSchematronParserCtxtPtr ctxt, xmlNodePtr node, int error,
 }
 
 /**
- * xmlSchematronVTypeErrMemory:
- * @node: a context node
- * @extra:  extra information
- *
  * Handle an out of memory condition
+ *
+ * @param ctxt  validation context
  */
 static void
 xmlSchematronVErrMemory(xmlSchematronValidCtxtPtr ctxt)
@@ -297,15 +280,12 @@ xmlSchematronVErrMemory(xmlSchematronValidCtxtPtr ctxt)
 }
 
 /**
- * xmlSchematronVErr:
- * @ctxt: the parsing context
- * @node: the context node
- * @error: the error code
- * @msg: the error message
- * @str1: extra data
- * @str2: extra data
- *
  * Handle a validation error
+ *
+ * @param ctxt  validation context
+ * @param error  the error code
+ * @param msg  the error message
+ * @param str1  extra data
  */
 static void LIBXML_ATTR_FORMAT(3,0)
 xmlSchematronVErr(xmlSchematronValidCtxtPtr ctxt, int error,
@@ -343,17 +323,15 @@ xmlSchematronVErr(xmlSchematronValidCtxtPtr ctxt, int error,
  ************************************************************************/
 
 /**
- * xmlSchematronAddTest:
- * @ctxt: the schema parsing context
- * @type:  the type of test
- * @rule:  the parent rule
- * @node:  the node hosting the test
- * @test: the associated test
- * @report: the associated report string
- *
  * Add a test to a schematron
  *
- * Returns the new pointer or NULL in case of error
+ * @param ctxt  the schema parsing context
+ * @param type  the type of test
+ * @param rule  the parent rule
+ * @param node  the node hosting the test
+ * @param test  the associated test
+ * @param report  the associated report string
+ * @returns the new pointer or NULL in case of error
  */
 static xmlSchematronTestPtr
 xmlSchematronAddTest(xmlSchematronParserCtxtPtr ctxt,
@@ -405,10 +383,9 @@ xmlSchematronAddTest(xmlSchematronParserCtxtPtr ctxt,
 }
 
 /**
- * xmlSchematronFreeTests:
- * @tests:  a list of tests
- *
  * Free a list of tests.
+ *
+ * @param tests  a list of tests
  */
 static void
 xmlSchematronFreeTests(xmlSchematronTestPtr tests) {
@@ -428,10 +405,9 @@ xmlSchematronFreeTests(xmlSchematronTestPtr tests) {
 }
 
 /**
- * xmlSchematronFreeLets:
- * @lets:  a list of let variables
- *
  * Free a list of let variables.
+ *
+ * @param lets  a list of let variables
  */
 static void
 xmlSchematronFreeLets(xmlSchematronLetPtr lets) {
@@ -449,16 +425,15 @@ xmlSchematronFreeLets(xmlSchematronLetPtr lets) {
 }
 
 /**
- * xmlSchematronAddRule:
- * @ctxt: the schema parsing context
- * @schema:  a schema structure
- * @node:  the node hosting the rule
- * @context: the associated context string
- * @report: the associated report string
- *
  * Add a rule to a schematron
  *
- * Returns the new pointer or NULL in case of error
+ * @param ctxt  the schema parsing context
+ * @param schema  a schema structure
+ * @param pat  a pattern
+ * @param node  the node hosting the rule
+ * @param context  the associated context string
+ * @param report  the associated report string
+ * @returns the new pointer or NULL in case of error
  */
 static xmlSchematronRulePtr
 xmlSchematronAddRule(xmlSchematronParserCtxtPtr ctxt, xmlSchematronPtr schema,
@@ -519,10 +494,9 @@ xmlSchematronAddRule(xmlSchematronParserCtxtPtr ctxt, xmlSchematronPtr schema,
 }
 
 /**
- * xmlSchematronFreeRules:
- * @rules:  a list of rules
- *
  * Free a list of rules.
+ *
+ * @param rules  a list of rules
  */
 static void
 xmlSchematronFreeRules(xmlSchematronRulePtr rules) {
@@ -546,15 +520,13 @@ xmlSchematronFreeRules(xmlSchematronRulePtr rules) {
 }
 
 /**
- * xmlSchematronAddPattern:
- * @ctxt: the schema parsing context
- * @schema:  a schema structure
- * @node:  the node hosting the pattern
- * @id: the id or name of the pattern
- *
  * Add a pattern to a schematron
  *
- * Returns the new pointer or NULL in case of error
+ * @param ctxt  the schema parsing context
+ * @param schema  a schema structure
+ * @param node  the node hosting the pattern
+ * @param name  the name of the pattern
+ * @returns the new pointer or NULL in case of error
  */
 static xmlSchematronPatternPtr
 xmlSchematronAddPattern(xmlSchematronParserCtxtPtr ctxt,
@@ -586,10 +558,9 @@ xmlSchematronAddPattern(xmlSchematronParserCtxtPtr ctxt,
 }
 
 /**
- * xmlSchematronFreePatterns:
- * @patterns:  a list of patterns
- *
  * Free a list of patterns.
+ *
+ * @param patterns  a list of patterns
  */
 static void
 xmlSchematronFreePatterns(xmlSchematronPatternPtr patterns) {
@@ -605,12 +576,10 @@ xmlSchematronFreePatterns(xmlSchematronPatternPtr patterns) {
 }
 
 /**
- * xmlSchematronNewSchematron:
- * @ctxt:  a schema validation context
- *
  * Allocate a new Schematron structure.
  *
- * Returns the newly allocated structure or NULL in case or error
+ * @param ctxt  a schema validation context
+ * @returns the newly allocated structure or NULL in case or error
  */
 static xmlSchematronPtr
 xmlSchematronNewSchematron(xmlSchematronParserCtxtPtr ctxt)
@@ -630,13 +599,12 @@ xmlSchematronNewSchematron(xmlSchematronParserCtxtPtr ctxt)
 }
 
 /**
- * xmlSchematronFree:
- * @schema:  a schema structure
- *
  * Deallocate a Schematron structure.
+ *
+ * @param schema  a schema structure
  */
 void
-xmlSchematronFree(xmlSchematronPtr schema)
+xmlSchematronFree(xmlSchematron *schema)
 {
     if (schema == NULL)
         return;
@@ -654,15 +622,13 @@ xmlSchematronFree(xmlSchematronPtr schema)
 }
 
 /**
- * xmlSchematronNewParserCtxt:
- * @URL:  the location of the schema
- *
  * Create an XML Schematrons parse context for that file/resource expected
  * to contain an XML Schematrons file.
  *
- * Returns the parser context or NULL in case of error
+ * @param URL  the location of the schema
+ * @returns the parser context or NULL in case of error
  */
-xmlSchematronParserCtxtPtr
+xmlSchematronParserCtxt *
 xmlSchematronNewParserCtxt(const char *URL)
 {
     xmlSchematronParserCtxtPtr ret;
@@ -693,16 +659,14 @@ xmlSchematronNewParserCtxt(const char *URL)
 }
 
 /**
- * xmlSchematronNewMemParserCtxt:
- * @buffer:  a pointer to a char array containing the schemas
- * @size:  the size of the array
- *
  * Create an XML Schematrons parse context for that memory buffer expected
  * to contain an XML Schematrons file.
  *
- * Returns the parser context or NULL in case of error
+ * @param buffer  a pointer to a char array containing the schemas
+ * @param size  the size of the array
+ * @returns the parser context or NULL in case of error
  */
-xmlSchematronParserCtxtPtr
+xmlSchematronParserCtxt *
 xmlSchematronNewMemParserCtxt(const char *buffer, int size)
 {
     xmlSchematronParserCtxtPtr ret;
@@ -731,16 +695,14 @@ xmlSchematronNewMemParserCtxt(const char *buffer, int size)
 }
 
 /**
- * xmlSchematronNewDocParserCtxt:
- * @doc:  a preparsed document tree
- *
  * Create an XML Schematrons parse context for that document.
  * NB. The document may be modified during the parsing process.
  *
- * Returns the parser context or NULL in case of error
+ * @param doc  a preparsed document tree
+ * @returns the parser context or NULL in case of error
  */
-xmlSchematronParserCtxtPtr
-xmlSchematronNewDocParserCtxt(xmlDocPtr doc)
+xmlSchematronParserCtxt *
+xmlSchematronNewDocParserCtxt(xmlDoc *doc)
 {
     xmlSchematronParserCtxtPtr ret;
 
@@ -770,13 +732,12 @@ xmlSchematronNewDocParserCtxt(xmlDocPtr doc)
 }
 
 /**
- * xmlSchematronFreeParserCtxt:
- * @ctxt:  the schema parser context
- *
  * Free the resources associated to the schema parser context
+ *
+ * @param ctxt  the schema parser context
  */
 void
-xmlSchematronFreeParserCtxt(xmlSchematronParserCtxtPtr ctxt)
+xmlSchematronFreeParserCtxt(xmlSchematronParserCtxt *ctxt)
 {
     if (ctxt == NULL)
         return;
@@ -793,12 +754,11 @@ xmlSchematronFreeParserCtxt(xmlSchematronParserCtxtPtr ctxt)
 
 #if 0
 /**
- * xmlSchematronPushInclude:
- * @ctxt:  the schema parser context
- * @doc:  the included document
- * @cur:  the current include node
- *
  * Add an included document
+ *
+ * @param ctxt  the schema parser context
+ * @param doc  the included document
+ * @param cur  the current include node
  */
 static void
 xmlSchematronPushInclude(xmlSchematronParserCtxtPtr ctxt,
@@ -832,12 +792,10 @@ xmlSchematronPushInclude(xmlSchematronParserCtxtPtr ctxt,
 }
 
 /**
- * xmlSchematronPopInclude:
- * @ctxt:  the schema parser context
- *
  * Pop an include level. The included document is being freed
  *
- * Returns the node immediately following the include or NULL if the
+ * @param ctxt  the schema parser context
+ * @returns the node immediately following the include or NULL if the
  *         include list was empty.
  */
 static xmlNodePtr
@@ -861,12 +819,11 @@ xmlSchematronPopInclude(xmlSchematronParserCtxtPtr ctxt)
 #endif
 
 /**
- * xmlSchematronAddNamespace:
- * @ctxt:  the schema parser context
- * @prefix:  the namespace prefix
- * @ns:  the namespace name
- *
  * Add a namespace definition in the context
+ *
+ * @param ctxt  the schema parser context
+ * @param prefix  the namespace prefix
+ * @param ns  the namespace name
  */
 static void
 xmlSchematronAddNamespace(xmlSchematronParserCtxtPtr ctxt,
@@ -905,11 +862,10 @@ xmlSchematronAddNamespace(xmlSchematronParserCtxtPtr ctxt,
 }
 
 /**
- * xmlSchematronParseTestReportMsg:
- * @ctxt:  the schema parser context
- * @con:  the assert or report node
- *
  * Format the message content of the assert or report test
+ *
+ * @param ctxt  the schema parser context
+ * @param con  the assert or report node
  */
 static void
 xmlSchematronParseTestReportMsg(xmlSchematronParserCtxtPtr ctxt, xmlNodePtr con)
@@ -955,11 +911,11 @@ xmlSchematronParseTestReportMsg(xmlSchematronParserCtxtPtr ctxt, xmlNodePtr con)
 }
 
 /**
- * xmlSchematronParseRule:
- * @ctxt:  a schema validation context
- * @rule:  the rule node
- *
  * parse a rule element
+ *
+ * @param ctxt  a schema validation context
+ * @param pattern  a pattern
+ * @param rule  the rule node
  */
 static void
 xmlSchematronParseRule(xmlSchematronParserCtxtPtr ctxt,
@@ -1123,11 +1079,10 @@ xmlSchematronParseRule(xmlSchematronParserCtxtPtr ctxt,
 }
 
 /**
- * xmlSchematronParsePattern:
- * @ctxt:  a schema validation context
- * @pat:  the pattern node
- *
  * parse a pattern element
+ *
+ * @param ctxt  a schema validation context
+ * @param pat  the pattern node
  */
 static void
 xmlSchematronParsePattern(xmlSchematronParserCtxtPtr ctxt, xmlNodePtr pat)
@@ -1172,13 +1127,11 @@ xmlSchematronParsePattern(xmlSchematronParserCtxtPtr ctxt, xmlNodePtr pat)
 
 #if 0
 /**
- * xmlSchematronLoadInclude:
- * @ctxt:  a schema validation context
- * @cur:  the include element
- *
  * Load the include document, Push the current pointer
  *
- * Returns the updated node pointer
+ * @param ctxt  a schema validation context
+ * @param cur  the include element
+ * @returns the updated node pointer
  */
 static xmlNodePtr
 xmlSchematronLoadInclude(xmlSchematronParserCtxtPtr ctxt, xmlNodePtr cur)
@@ -1238,17 +1191,15 @@ done:
 #endif
 
 /**
- * xmlSchematronParse:
- * @ctxt:  a schema validation context
- *
  * parse a schema definition resource and build an internal
  * XML Schema structure which can be used to validate instances.
  *
- * Returns the internal XML Schematron structure built from the resource or
+ * @param ctxt  a schema validation context
+ * @returns the internal XML Schematron structure built from the resource or
  *         NULL in case of error
  */
-xmlSchematronPtr
-xmlSchematronParse(xmlSchematronParserCtxtPtr ctxt)
+xmlSchematron *
+xmlSchematronParse(xmlSchematronParserCtxt *ctxt)
 {
     xmlSchematronPtr ret = NULL;
     xmlDocPtr doc;
@@ -1413,36 +1364,23 @@ exit:
  *                                                                      *
  ************************************************************************/
 
-static xmlNodePtr
+static xmlXPathObjectPtr
 xmlSchematronGetNode(xmlSchematronValidCtxtPtr ctxt,
                      xmlNodePtr cur, const xmlChar *xpath) {
-    xmlNodePtr node = NULL;
-    xmlXPathObjectPtr ret;
-
     if ((ctxt == NULL) || (cur == NULL) || (xpath == NULL))
         return(NULL);
 
     ctxt->xctxt->doc = cur->doc;
     ctxt->xctxt->node = cur;
-    ret = xmlXPathEval(xpath, ctxt->xctxt);
-    if (ret == NULL)
-        return(NULL);
-
-    if ((ret->type == XPATH_NODESET) &&
-        (ret->nodesetval != NULL) && (ret->nodesetval->nodeNr > 0))
-        node = ret->nodesetval->nodeTab[0];
-
-    xmlXPathFreeObject(ret);
-    return(node);
+    return(xmlXPathEval(xpath, ctxt->xctxt));
 }
 
 /**
- * xmlSchematronReportOutput:
- * @ctxt: the validation context
- * @cur: the current node tested
- * @msg: the message output
- *
  * Output part of the report to whatever channel the user selected
+ *
+ * @param ctxt  the validation context
+ * @param cur  the current node tested
+ * @param msg  the message output
  */
 static void
 xmlSchematronReportOutput(xmlSchematronValidCtxtPtr ctxt ATTRIBUTE_UNUSED,
@@ -1453,14 +1391,12 @@ xmlSchematronReportOutput(xmlSchematronValidCtxtPtr ctxt ATTRIBUTE_UNUSED,
 }
 
 /**
- * xmlSchematronFormatReport:
- * @ctxt:  the validation context
- * @test: the test node
- * @cur: the current node tested
- *
  * Build the string being reported to the user.
  *
- * Returns a report string or NULL in case of error. The string needs
+ * @param ctxt  the validation context
+ * @param test  the test node
+ * @param cur  the current node tested
+ * @returns a report string or NULL in case of error. The string needs
  *         to be deallocated by the caller
  */
 static xmlChar *
@@ -1479,25 +1415,40 @@ xmlSchematronFormatReport(xmlSchematronValidCtxtPtr ctxt,
             (child->type == XML_CDATA_SECTION_NODE))
             ret = xmlStrcat(ret, child->content);
         else if (IS_SCHEMATRON(child, "name")) {
+            xmlXPathObject *obj = NULL;
             xmlChar *path;
 
             path = xmlGetNoNsProp(child, BAD_CAST "path");
 
             node = cur;
             if (path != NULL) {
-                node = xmlSchematronGetNode(ctxt, cur, path);
-                if (node == NULL)
-                    node = cur;
+                obj = xmlSchematronGetNode(ctxt, cur, path);
+                if ((obj != NULL) &&
+                    (obj->type == XPATH_NODESET) &&
+                    (obj->nodesetval != NULL) &&
+                    (obj->nodesetval->nodeNr > 0))
+                    node = obj->nodesetval->nodeTab[0];
                 xmlFree(path);
             }
 
-            if ((node->ns == NULL) || (node->ns->prefix == NULL))
-                ret = xmlStrcat(ret, node->name);
-            else {
-                ret = xmlStrcat(ret, node->ns->prefix);
-                ret = xmlStrcat(ret, BAD_CAST ":");
-                ret = xmlStrcat(ret, node->name);
+            switch (node->type) {
+                case XML_ELEMENT_NODE:
+                case XML_ATTRIBUTE_NODE:
+                    if ((node->ns == NULL) || (node->ns->prefix == NULL))
+                        ret = xmlStrcat(ret, node->name);
+                    else {
+                        ret = xmlStrcat(ret, node->ns->prefix);
+                        ret = xmlStrcat(ret, BAD_CAST ":");
+                        ret = xmlStrcat(ret, node->name);
+                    }
+                    break;
+
+                /* TODO: handle other node types */
+                default:
+                    break;
             }
+
+            xmlXPathFreeObject(obj);
         } else if (IS_SCHEMATRON(child, "value-of")) {
             xmlChar *select;
             xmlXPathObjectPtr eval;
@@ -1505,6 +1456,11 @@ xmlSchematronFormatReport(xmlSchematronValidCtxtPtr ctxt,
             select = xmlGetNoNsProp(child, BAD_CAST "select");
             comp = xmlXPathCtxtCompile(ctxt->xctxt, select);
             eval = xmlXPathCompiledEval(comp, ctxt->xctxt);
+            if (eval == NULL) {
+                xmlXPathFreeCompExpr(comp);
+                xmlFree(select);
+                return ret;
+            }
 
             switch (eval->type) {
             case XPATH_NODESET: {
@@ -1582,14 +1538,14 @@ xmlSchematronFormatReport(xmlSchematronValidCtxtPtr ctxt,
 }
 
 /**
- * xmlSchematronReportSuccess:
- * @ctxt:  the validation context
- * @test: the compiled test
- * @cur: the current node tested
- * @success: boolean value for the result
- *
  * called from the validation engine when an assert or report test have
  * been done.
+ *
+ * @param ctxt  the validation context
+ * @param test  the compiled test
+ * @param cur  the current node tested
+ * @param pattern  a pattern
+ * @param success  boolean value for the result
  */
 static void
 xmlSchematronReportSuccess(xmlSchematronValidCtxtPtr ctxt,
@@ -1672,11 +1628,10 @@ xmlSchematronReportSuccess(xmlSchematronValidCtxtPtr ctxt,
 }
 
 /**
- * xmlSchematronReportPattern:
- * @ctxt:  the validation context
- * @pattern: the current pattern
- *
  * called from the validation engine when starting to check a pattern
+ *
+ * @param ctxt  the validation context
+ * @param pattern  the current pattern
  */
 static void
 xmlSchematronReportPattern(xmlSchematronValidCtxtPtr ctxt,
@@ -1705,15 +1660,14 @@ xmlSchematronReportPattern(xmlSchematronValidCtxtPtr ctxt,
  ************************************************************************/
 
 /**
- * xmlSchematronSetValidStructuredErrors:
- * @ctxt:  a Schematron validation context
- * @serror:  the structured error function
- * @ctx: the functions context
- *
  * Set the structured error callback
+ *
+ * @param ctxt  a Schematron validation context
+ * @param serror  the structured error function
+ * @param ctx  the functions context
  */
 void
-xmlSchematronSetValidStructuredErrors(xmlSchematronValidCtxtPtr ctxt,
+xmlSchematronSetValidStructuredErrors(xmlSchematronValidCtxt *ctxt,
                                       xmlStructuredErrorFunc serror, void *ctx)
 {
     if (ctxt == NULL)
@@ -1725,20 +1679,21 @@ xmlSchematronSetValidStructuredErrors(xmlSchematronValidCtxtPtr ctxt,
 }
 
 /**
- * xmlSchematronNewValidCtxt:
- * @schema:  a precompiled XML Schematrons
- * @options: a set of xmlSchematronValidOptions
- *
  * Create an XML Schematrons validation context based on the given schema.
  *
- * Returns the validation context or NULL in case of error
+ * @param schema  a precompiled XML Schematrons
+ * @param options  a set of xmlSchematronValidOptions
+ * @returns the validation context or NULL in case of error
  */
-xmlSchematronValidCtxtPtr
-xmlSchematronNewValidCtxt(xmlSchematronPtr schema, int options)
+xmlSchematronValidCtxt *
+xmlSchematronNewValidCtxt(xmlSchematron *schema, int options)
 {
     int i;
     xmlSchematronValidCtxtPtr ret;
 
+    if (schema == NULL)
+        return(NULL);
+
     ret = (xmlSchematronValidCtxtPtr) xmlMalloc(sizeof(xmlSchematronValidCtxt));
     if (ret == NULL) {
         xmlSchematronVErrMemory(NULL);
@@ -1765,13 +1720,12 @@ xmlSchematronNewValidCtxt(xmlSchematronPtr schema, int options)
 }
 
 /**
- * xmlSchematronFreeValidCtxt:
- * @ctxt:  the schema validation context
- *
  * Free the resources associated to the schema validation context
+ *
+ * @param ctxt  the schema validation context
  */
 void
-xmlSchematronFreeValidCtxt(xmlSchematronValidCtxtPtr ctxt)
+xmlSchematronFreeValidCtxt(xmlSchematronValidCtxt *ctxt)
 {
     if (ctxt == NULL)
         return;
@@ -1818,15 +1772,14 @@ xmlSchematronNextNode(xmlNodePtr cur) {
 }
 
 /**
- * xmlSchematronRunTest:
- * @ctxt:  the schema validation context
- * @test:  the current test
- * @instance:  the document instance tree
- * @cur:  the current node in the instance
- *
  * Validate a rule against a tree instance at a given position
  *
- * Returns 1 in case of success, 0 if error and -1 in case of internal error
+ * @param ctxt  the schema validation context
+ * @param test  the current test
+ * @param instance  the document instance tree
+ * @param cur  the current node in the instance
+ * @param pattern  a pattern
+ * @returns 1 in case of success, 0 if error and -1 in case of internal error
  */
 static int
 xmlSchematronRunTest(xmlSchematronValidCtxtPtr ctxt,
@@ -1880,15 +1833,14 @@ xmlSchematronRunTest(xmlSchematronValidCtxtPtr ctxt,
 }
 
 /**
- * xmlSchematronRegisterVariables:
- * @ctxt:  the schema validation context
- * @let:  the list of let variables
- * @instance:  the document instance tree
- * @cur:  the current node
+ * Registers a list of let variables to the current context of `cur`
  *
- * Registers a list of let variables to the current context of @cur
- *
- * Returns -1 in case of errors, otherwise 0
+ * @param vctxt  the schema validation context
+ * @param ctxt  an XPath context
+ * @param let  the list of let variables
+ * @param instance  the document instance tree
+ * @param cur  the current node
+ * @returns -1 in case of errors, otherwise 0
  */
 static int
 xmlSchematronRegisterVariables(xmlSchematronValidCtxtPtr vctxt,
@@ -1919,13 +1871,12 @@ xmlSchematronRegisterVariables(xmlSchematronValidCtxtPtr vctxt,
 }
 
 /**
- * xmlSchematronUnregisterVariables:
- * @ctxt:  the schema validation context
- * @let:  the list of let variables
- *
  * Unregisters a list of let variables from the context
  *
- * Returns -1 in case of errors, otherwise 0
+ * @param vctxt  the schema validation context
+ * @param ctxt  an XPath context
+ * @param let  the list of let variables
+ * @returns -1 in case of errors, otherwise 0
  */
 static int
 xmlSchematronUnregisterVariables(xmlSchematronValidCtxtPtr vctxt,
@@ -1944,17 +1895,15 @@ xmlSchematronUnregisterVariables(xmlSchematronValidCtxtPtr vctxt,
 }
 
 /**
- * xmlSchematronValidateDoc:
- * @ctxt:  the schema validation context
- * @instance:  the document instance tree
- *
  * Validate a tree instance against the schematron
  *
- * Returns 0 in case of success, -1 in case of internal error
+ * @param ctxt  the schema validation context
+ * @param instance  the document instance tree
+ * @returns 0 in case of success, -1 in case of internal error
  *         and an error count otherwise.
  */
 int
-xmlSchematronValidateDoc(xmlSchematronValidCtxtPtr ctxt, xmlDocPtr instance)
+xmlSchematronValidateDoc(xmlSchematronValidCtxt *ctxt, xmlDoc *instance)
 {
     xmlNodePtr cur, root;
     xmlSchematronPatternPtr pattern;
